; ---------------------------------------------------------------------------------------------------
; ACESSANDO UMA MEMORIA RAM UM61256FK                                                      23/09/2015
;
; STATUS: FUNCIONA
;
; Grava e le uma string de caracteres numa memoria RAM externa UM61256FK.
;
; O bus de endereco sera conectado a 2 shift registers 74LS164N que receberao o endereco da
; memoria de modo serial para economizar portas de I/O do PIC.
;
; O bus de dados (pinos 11, 12, 13, 15, 16, 17, 18 e 19 da RAM) serao conectados a uma das 
; portas de I/O (8 bits).
;
; As definicoes de I/O, os registradores e simbolos usados para controlar a RAM , assim
; como as funcoes de controle para inicializar, ler e gravar a RAM  estao no arquivo 
; 'ramum16256_services.inc'.
;
; HARDWARE
; RA0 ........................... Endereco - shift register LSB   pino   1
; RA1 ........................... Clock    - shift register LSB   pino   8
; RA2 ........................... CE       - chip select da RAM   pino  20
; RA3 ........................... OE       - output eneble da RAM pino  22
; RA4 ........................... WE       - write enable da RAM  pino  27
; RB0 - RB7 ..................... Dados    - bus de dados da RAM  pinos 11,12,13,15,16,17,18 e 19 
; RD1 ........................... LCD Data
; RD2 ........................... LCD Clock
;
; Ver esquema de conexoes entre a memoria e os shift-registers na pasta "ESQUEMA\um61256fk_pic.png"
; deste projeto.
;
; 
; O LCD usado e' do tipo HD44780 controlado serialmente por um shift-register SN74LS164 ou 4015.
;
;
; ---------------------------------------------------------------------------------------------------
;
	LIST P=18F4620
	#include <P18F4620.INC>
;
; ---------------------------------------------------------------------------------------------
; CONFIGURATION BITS ou FUSES
; ---------------------------------------------------------------------------------------------
    CONFIG WDT    = OFF
	CONFIG MCLRE  = ON
	CONFIG DEBUG  = OFF
	CONFIG LVP    = OFF 
;    CONFIG OSC    = INTIO67      ; oscilador interno (pode ser tambem INTIO7)
    CONFIG OSC    = HS         ; oscilador a cristal externo
    CONFIG CP0    = OFF
    CONFIG CP1    = OFF
    CONFIG CP2    = OFF
    CONFIG CP3    = OFF
    CONFIG CPB    = OFF
    CONFIG CPD    = OFF
    CONFIG WRT0   = OFF
    CONFIG WRT1   = OFF
    CONFIG WRT2   = OFF
    CONFIG WRT3   = OFF
    CONFIG WRTC   = OFF
    CONFIG WRTB   = OFF
    CONFIG WRTD   = OFF
; ---------------------------------------------------------------------------------------------
    #include "FAMILIAS.mac"

    FAMILIAS
 
; ---------------------------------------------------------------------------------------------
             UDATA   0x080
; ---------------------------------------------------------------------------------------------
; Registradores para servicos de delay
; ---------------------------------------------------------------------------------------------
delay_temp   RES     1
CLOCK        equ     20000000

; ---------------------------------------------------------------------------------------------
; Registradores para servicos de LCD
; ---------------------------------------------------------------------------------------------
SR_1         RES          1             ; registrador auxiliar 1
SR_2         RES          1             ; registrador auxiliar 2
SR_3         RES          1             ; registrador auxiliar 3
SR_4         RES          1             ; registrador auxiliar 4
SR_5         RES          1             ; registrador auxiliar 5
LCD_COMMAND  RES          1             ; buffer de comando e dados
SR_TIPO      RES          1             ; tipo de LCD
SR_CLK       equ          2             ; em TRISx este bit deve ser saida
SR_DAT       equ          1             ; em TRISx este bit deve ser saida
SR_CHARS     equ          0             ; 0 - caracteres 5x7    1 - caracteres 5x10
#define      SR_TRIS      TRISD         ; I/O
#define      SR_DATA      LATD,SR_DAT   ; DATA
#define      SR_CLOCK     LATD,SR_CLK   ; CLOCK

; ---------------------------------------------------------------------------------------------
; Registradores para servicos de string
; ---------------------------------------------------------------------------------------------
str_dlm       RES     1
str_reg1      RES     1
str_reg2      RES     1
str_reg3      RES     1
str_reg4      RES     1
str_reg5      RES     1
str_reg6      RES     1
str_ptr       RES     1
str_buffer    EQU     0x800 

; ---------------------------------------------------------------------------------------------
; Registradores para servicos de RAM UM61256FK
; ---------------------------------------------------------------------------------------------
RAMREG4      RES     1                 ; registrador auxiliar
RAMREG3      RES     1                 ; registrador auxiliar
RAMREG2      RES     1                 ; registrador auxiliar
RAMREG1      RES     1                 ; registrador auxiliar
RAMREG0      RES     1                 ; registrador auxiliar
RAM_ADDR1    RES     1                 ; RAM DATA Address MSB (7 bits)
RAM_ADDR0    RES     1                 ; RAM DATA Address LSB (8 bits)
RAMIO_TRIS   EQU     TRISB             ; RAM I/O
RAMIO_PORT   EQU     PORTB             ; RAM I/O Port
sr_tris      EQU     TRISA             ; serial data I/O
sr_port      EQU     LATA              ; serial data Port
sr_data      EQU     0                 ; serial data bit
sr_clock     EQU     1                 ; serial clock bit
RAMCT_TRIS   EQU     TRISA             ; RAM CONTROL I/O   
RAMCT_PORT   EQU     LATA              ; RAM CONTROL Port
RAM_CE       EQU     2                 ; RAM Chip Enable   (Active Low) 
RAM_OE       EQU     3                 ; RAM Output Enable (Active Low)
RAM_WE       EQU     4                 ; RAM Write Enable  (Active Low)

; ---------------------------------------------------------------------------------------------
; Registradores de uso geral
; ---------------------------------------------------------------------------------------------
msgbuff      RES     .32               ; buffer 
;
             #include "GPR.inc"        ; registradores de trabalho




; ---------------------------------------------------------------------------------------------
; Vetores
; ---------------------------------------------------------------------------------------------
pwron     CODE  0x0000
          goto  inicio                 ; inicio
intrf     CODE  0x008
          retfie                       ; nao ha interrupcoes neste programa

; ---------------------------------------------------------------------------------------------
; Inicio do programa
; ---------------------------------------------------------------------------------------------
MAIN      CODE

          #include "delay_services.inc"          ; servicos de tempo de retardo
          #include "lcd_services.inc"            ; servicos de LCD
          #include "string_services.inc"         ; servicos de string
          #include "ramum61256_services.inc"     ; servicos de acesso a RAM UM61256FK

          LCD_LINES 0216                         ; define o tipo de LCD
                                                 ; 2 linhas por 16 colunas

; ---------------------------------------------------------------------------------------------
; Inicializacao geral do PIC
; ---------------------------------------------------------------------------------------------
inicio    movlw   0x0F                           ; mascara
          movwf   ADCON1                         ; todos as portas sao digitais

          movlw   0x00                           ; W = 0x00
          movwf   LATA                           ; PORTA=0x00
          movwf   LATB                           ; PORTB=0x00
          movwf   LATC                           ; PORTC=0x00
          movwf   LATD                           ; PORTD=0x00
          movwf   LATE                           ; PORTE=0x00

          movwf   TRISA                          ; PORTA como saida
          movwf   TRISB                          ; PORTB como saida
          movwf   TRISC                          ; PORTC como saida
          movwf   TRISD                          ; PORTD como saida
          movwf   TRISE                          ; PORTE como saida
;
          DLMSET  0x0D                           ; define o delimitador de strings
;
          call    SR_4BITS                       ; configuracao do LCD
          call    SR_LCDCLEAR                    ; limpa o LCD
;
          call    RAM_INIT                       ; configura controles da RAM externa

; ---------------------------------------------------------------------------------------------
; Gera strings na memoria RAM do PIC e grava as mesmas na memoria RAM externa
; ---------------------------------------------------------------------------------------------
msg001    equ     0x0100                         ; endereco de 'msg001' na RAM externa
msg002    equ     0x0600                         ; endereco de 'msg002' na RAM externa

          COPYSTR msgbuff, " RAM  UM61256FK "    ; gera a string de 16 bytes na RAM do PIC

          RAM_MVC  msgbuff+.1, .16, msg001       ; move a string de 16 bytes da RAM do PIC 
                                                 ; para o endereco 'msg001' da RAM externa

          COPYSTR msgbuff, " Ola Pessoal!!! "    ; gera uma string de 16 bytes na RAM do PIC

          RAM_MVC  msgbuff+.1, .16, msg002       ; move a string de 16 bytes da RAM do PIC
                                                 ; para o endereco 'msg002' da RAM externa

          ; -----------------------------------------------------------------------------------
          ; NOTA: 
          ; A macro RAM_MVC acima usa 'msgbuff+.1' porque na string gerada pela macro COPYSTR
          ; o primeiro byte e' usado para especificar o tamanho da string portanto o primeiro
          ; caracter comeca na posicao 2 da string, ou seja, deslocamento +1.
          ; -----------------------------------------------------------------------------------
  
; ---------------------------------------------------------------------------------------------
; Mostra no LCD as strings gravadas na RAM externa
; ---------------------------------------------------------------------------------------------
retry     SET_CURSOR .1,.1,"i"                   ; posiciona o cursor do LCD

          RAM_ADDRESS  msg001                    ; carrega registradores de endereco da RAM externa
                                                 ; com o endereco de 'msg001'

          call    showram                        ; vai mostrar o conteudo desse endereco


          SET_CURSOR .2,.1,"i"                   ; posiciona o cursor do LCD

          RAM_ADDRESS  msg002                    ; carrega registradores de endereco da RAM externa
                                                 ; com o endereco de 'msg002'

          call    showram                        ; vai mostrar o conteudo desse endereco

          movlw   .2                             ; 2 segundos
          call    tempo                          ; tempo para visualizacao no LCD
          call    SR_LCDCLEAR                    ; limpa o LCD
          movlw   .2                             ; 2 segundos
          call    tempo                          ; tempo com o LCD apagado

          goto    retry                          ; repete essa operacao indefinidamente

; ---------------------------------------------------------------------------------------------
; Funcao de exibicao de strings no LCD
; ---------------------------------------------------------------------------------------------
showram   movlw   .16                            ; tamanho da string
          movwf   R12                            ; carrega o contador
loop1     call    RAM_GETCHAR                    ; le um caracter da string enderecada
          call    SR_SENDW                       ; mostra o caracter no LCD
          call    RAM_INC                        ; avanca um byte na RAM externa
          decfsz  R12, f                         ; fim da string???
          goto    loop1                          ; NAO --> vai ler o proximo caracter
          return

; ---------------------------------------------------------------------------------------------
; Funcao de tempo em segundos
;
; Entrada: W deve conter o numero de segundos
; ---------------------------------------------------------------------------------------------
tempo     movwf   R15
          rlcf    R15, f
          rlcf    R15, f
t1        movlw   .250
          call    delay_ms
          decfsz  R15, f
          goto    t1
          return
; ---------------------------------------------------------------------------------------------
; ---------------------------------------------------------------------------------------------
; ---------------------------------------------------------------------------------------------
          END
