; ------------------------------------------------------------------------
; Servicos de Acesso a Memoria RAM Externa UM61256FK            22/09/2015
; ------------------------------------------------------------------------
;
; O seguinte arquivo deve ser inserido no programa:
; #include "ramum61256_services.inc"   ; funcoes de acesso
;
; Os seguintes simbolos e registradores devem ser inseridos no programa:
;RAMREG4      RES     1                 ; registrador auxiliar
;RAMREG3      RES     1                 ; registrador auxiliar
;RAMREG2      RES     1                 ; registrador auxiliar
;RAMREG1      RES     1                 ; registrador auxiliar
;RAMREG0      RES     1                 ; registrador auxiliar
;RAM_ADDR1    RES     1                 ; RAM DATA Address MSB (7 bits)
;RAM_ADDR0    RES     1                 ; RAM DATA Address LSB (8 bits)
;RAMIO_TRIS   EQU     TRISB             ; RAM I/O
;RAMIO_PORT   EQU     PORTB             ; RAM I/O Port
;sr_tris      EQU     TRISA             ; serial data I/O
;sr_port      EQU     LATA              ; serial data Port
;sr_data      EQU     0                 ; serial data bit
;sr_clock     EQU     1                 ; serial clock bit
;RAMCT_TRIS   EQU     TRISA             ; RAM CONTROL I/O   
;RAMCT_PORT   EQU     LATA              ; RAM CONTROL Port
;RAM_CE       EQU     2                 ; RAM Chip Enable   (Active Low) 
;RAM_OE       EQU     3                 ; RAM Output Enable (Active Low)
;RAM_WE       EQU     4                 ; RAM Write Enable  (Active Low)
;
; NOTAS:
; 1 - Os valores dos simbolos acima, definidos pela diretiva "EQU", podem 
;     ser modificados conforme as necessidades do projeto.
;
; ------------------------------------------------------------------------
; A RAM UM61256FK externa ao microcontrolador sera acessada pelas
; funcoes aqui codificadas:
;
; RAM_INIT ............. inicializa os pinos de controle da RAM.
;                        Zera os registradores de endereco RAM_ADDR0:RAM_ADDR1.
;                        Posiciona o bus de endereco da RAM na posicao zero.
;                        Esta funcao deve ser usada uma unica vez no inicio do 
;                        programa.
;
; RAM_SETADDR .......... envia ao bus de endereco da RAM o endereco contido
;                        no par de registradores RAM_ADDR0:RAM_ADDR1.
;
; RAM_PUTCHAR .......... grava o caracter contido no registrador W no endereco
;                        corrente no par de registradores RAM_ADDR0:RAM_ADDR1.
;
; RAM_GETCHAR .......... Le e armazena no registrador W, o caracter apontado
;                        pelo endereco corrente no par de registradores 
;                        RAM_ADDR0:RAM_ADDR1.
;
; RAM_INC .............. Incrementa de 1 o endereco contido no par de
;                        registradores RAM_ADDR0:RAM_ADDR1 e envia o novo
;                        endereco ao bus de endereco da RAM.
;
; RAM_MVC .............. Esta Macro envia uma string da RAM do PIC para
;                        RAM externa.
;
; RAM_ADDRESS .......... Esta Macro carrega o par de registradores
;                        RAM_ADDR1:RAM_ADDR0 com o endereco desejado e
;                        envia-o para o bus de endereco da RAM.
;
; ------------------------------------------------------------------------

; ------------------------------------------------------------------------
; Funcoes exclusivamente de uso interno.                        22/09/2015
; O programador nunca deve usar essas funcoes.
; ------------------------------------------------------------------------
#define      RAM_CHIP_ENABLE    bcf  RAMCT_PORT, RAM_CE
#define      RAM_CHIP_DISABLE   bsf  RAMCT_PORT, RAM_CE
#define      RAM_OUTPUT_ENABLE  bcf  RAMCT_PORT, RAM_OE
#define      RAM_OUTPUT_DISABLE bsf  RAMCT_PORT, RAM_OE
#define      RAM_WRITE_ENABLE   bcf  RAMCT_PORT, RAM_WE
#define      RAM_WRITE_DISABLE  bsf  RAMCT_PORT, RAM_WE

; ------------------------------------------------------------------------
; MACRO - RAM_MVC                                               23/09/2015
;
; Move <numero> caracteres <origem> da RAM do microcontrolador, para
; o endereco <destino> da memoria RAM externa UM61256FK. 
; Nao ha controle de estouro do endereco. Se isso acontecer, o par
; RAM_ADDR1:RAM_ADDR0 voltara a apontar para 0x0000.
; ------------------------------------------------------------------------
RAM_MVC        MACRO   origem, numero, destino
 ifndef _numq
      variable _numq
_numq set 0
 endif
_numq set _numq + 1
               movlw   high(destino)
               movwf   RAM_ADDR1
               movlw   low(destino)
               movwf   RAM_ADDR0
               movlw   numero
               movwf   RAMREG4
               lfsr    FSR0, origem
_um61#v(_numq) movf    POSTINC0, w
               call    RAM_PUTCHAR
               call    RAM_INC
               decfsz  RAMREG4, f
               goto    _um61#v(_numq)
               ENDM
; ------------------------------------------------------------------------
; MACRO - RAM_ADDRESS                                           23/09/2015
;
; O par de registradores de endereco RAM_ADDR1:RAM_ADDR0 sera carregado
; com o endereco especificado no parametro <endereco> e o novo valor do
; endereco sera enviado ao bus de endereco da RAM.
;
; ------------------------------------------------------------------------
RAM_ADDRESS    MACRO   endereco
               movlw   high(endereco)            ; MSB do endereco
               movwf   RAM_ADDR1                 ; 
               movlw   low(endereco)             ; LSB do endereco
               movwf   RAM_ADDR0                 ;
               call    RAM_SETADDR               ; especifica o endereco da RAM
               ENDM
; ------------------------------------------------------------------------
; Incrementa o endereco da RAM                                  23/09/2015
;
; O par de registradores de endereco RAM_ADDR1:RAM_ADDR0 sera incrementado
; de uma unidade e o novo endereco sera enviado ao bus de endereco da RAM.
;
; Entrada: Nao ha
;
; Saida  : O par RAM_ADDR1:RAM_ADDR0 sera incrementado uma unidade.
;
; ------------------------------------------------------------------------
RAM_INC        infsnz  RAM_ADDR0, f              ; inc LSB
               goto    _ovflsb                   ; estouro do LSB 
               call    RAM_SETADDR               ; especifica o endereco da RAM
               return                            ; return if not zero
_ovflsb        incf    RAM_ADDR1,f               ; inc MSB
               call    RAM_SETADDR               ; especifica o endereco da RAM
               return                            ; return
; ------------------------------------------------------------------------
; Inicializa a RAM                                              22/09/2015
;
; Esta funcao inicializa o par de registradores de endereco da RAM
; RAM_ADDR1:RAM_ADDR0 fazendo-os apontar para a posicao zero (0x0000)
; da memoria e envia esse valor para o bus de endereco da RAM.
; Alem disso, esta funcao desabilita a leitura e gravacao e habilita
; o chip de memoria para torna-lo disponivel para as outras funcoes.
; Recomenda-se usar essa funcao uma unica vez no inicio do programa.
;
;
; Entrada: Nao ha
;
; Saida  : Os flags da RAM WE, CE e OE serao desabilitados.
;          O par RAM_ADDR0:RAM_ADDR1 apontara para o endereco 0x0000. 
;
; ------------------------------------------------------------------------
RAM_INIT       RAM_CHIP_DISABLE                  ; desabilita a RAM
               movlw  .20                        ; milisegundos
               call   delay_ms                   ; estabilizacao
               RAM_WRITE_DISABLE                 ; desabilita gravacao
               movlw  .20                        ; milisegundos
               call   delay_ms                   ; estabilizacao
               RAM_OUTPUT_DISABLE                ; desabilita leitura
               movlw  .20                        ; milisegundos
               call   delay_ms                   ; estabilizacao
               clrf    RAM_ADDR0                 ; endereco LSB = 0x00
               clrf    RAM_ADDR1                 ; endereco MSB = 0x00
               RAM_CHIP_ENABLE                   ; habilita a RAM
               movlw  .20                        ; milisegundos
               call   delay_ms                   ; estabilizacao
               nop                               ; estabilizacao
               nop                               ; estabilizacao
               call    RAM_SETADDR               ; posiciona RAM no endereco 0x0000
               return                            ;
; ------------------------------------------------------------------------
; Especifica um endereco da RAM                                 22/09/2015
;
; Esta funcao envia, para o bus de endereco da RAM, os 15 bits de endereco  
; contidos no par de registradores de endereco RAM_ADDR0:RAM_ADDR1. 
;
; Entrada: O par RAM_ADDR0:RAM_ADDR1 contem o endereco desejado.
;
; Saida  : O endereco sera colocado no bus de endereco da RAM.
;
; ------------------------------------------------------------------------
RAM_SETADDR    movff   RAM_ADDR0, RAMREG0        ; pega LSB do endereco
               movff   RAM_ADDR1, RAMREG1        ; pega MSB do endereco
               bcf     sr_tris, sr_data          ; serial data como saida
               movlw   .16                       ; 16 bits a deslocar
               movwf   RAMREG2                   ; carrega contador de bits
RAM_SETADDR1   bcf     sr_port, sr_data          ; serial data = 0
               rlcf    RAMREG0, f                ; desloca um bit para esquerda
               rlcf    RAMREG1, f                ; desloca um bit para esquerda
               btfsc   STATUS, C                 ; MSB deslocado e' zero???
               bsf     sr_port, sr_data          ; NAO --> serial data = 1
               bsf     sr_port, sr_clock         ; levanta o clock do shift register
               nop                               ; estabilizar
               bcf     sr_port, sr_clock         ; abaixa o clock do shift register
               decfsz  RAMREG2, f                ; conta o bit enviado
               goto    RAM_SETADDR1              ; manda proximo bit
               return                            ; fim 
; ------------------------------------------------------------------------
; Grava um byte no endereco corrente da RAM                     22/09/2015
;
; Esta funcao envia o conteudo do registrador W para o endereco corrente
; no bus de endereco da memoria RAM.
;
; Entrada: W contem o byte a gravar.
;
; Saida  : O conteudo de W gravado no endereco corrente.
;
; ------------------------------------------------------------------------
RAM_PUTCHAR    movwf   RAMREG3                   ; salva caracher a enviar
               call    RAM_SETADDR               ; posiciona RAM no endereco 0x0000
               clrf    RAMIO_TRIS                ; I/O Data como saida
               movf    RAMREG3, w                ; pega o caracter a enviar
               movwf   RAMIO_PORT                ; envia para a RAM
               nop                               ; estabilizar
               RAM_WRITE_ENABLE                  ; habilita a gravacao na RAM
               nop                               ; estabilizar
               RAM_WRITE_DISABLE                 ; desabilita a gravacao na RAM
               clrf    RAMIO_PORT                ; limpa a porta de dados
               return                            ; fim
; ------------------------------------------------------------------------
; Le um byte no endereco corrente da RAM                        22/09/2015
;
; Esta funcao le o endereco corrente no bus de endereco da RAM e armazena
; o seu valor.
;
; Entrada: Nao ha
;
; Saida  : W contera o byte lido no endereco da RAM.
;
; ------------------------------------------------------------------------
RAM_GETCHAR    movlw   0xFF                      ; mascara de I/O Data 
               movwf   RAMIO_TRIS                ; I/O data como entrada
               call    RAM_SETADDR               ; posiciona RAM no endereco 0x0000
               RAM_OUTPUT_ENABLE                 ; habilita a leitura da RAM
               nop                               ; estabilizar
               movf    RAMIO_PORT, w             ; le a RAM
               nop                               ; estabilizar
               RAM_OUTPUT_DISABLE                ; desabilita a leitura da RAM
               return                            ; fim
